/*
    SDL - Simple DirectMedia Layer
    Copyright (C) 1997-2009 Sam Lantinga

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Sam Lantinga
    slouken@libsdl.org
*/
#include "SDL_config.h"

/* Being a null driver, there's no event stream. We just define stubs for
   most of the API. */

#include "SDL.h"
#include "../../events/SDL_sysevents.h"
#include "../../events/SDL_events_c.h"

#include "SDL_nullvideo.h"
#include "SDL_nullevents_c.h"

#if SDL_INPUT_LINUXEV
#include <linux/input.h>
#include <linux/limits.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#endif

extern int InputKeyData, InputKeyState;
static SDLKey keymapRemote[256];
static SDLKey keymapKeyboard[KEY_MAX];

#if SDL_INPUT_LINUXEV

#define MAX_EVDEV 32

static struct {
	int fd;
} evdevlist[MAX_EVDEV];

#define NBITS(x) ((((x)-1)/(sizeof(long) * 8))+1)
#define test_bit(nr, addr) \
	 (((1UL << ((nr) % (sizeof(long) * 8))) & ((addr)[(nr) / (sizeof(long) * 8)])) != 0)

static int IsMouseOrKeyboard(int fd) {
	unsigned long evbit[NBITS(EV_MAX)] = { 0 };
	unsigned long relbit[NBITS(REL_MAX)] = { 0 };
	unsigned long keybit[NBITS(KEY_MAX)] = { 0 };

	if ((ioctl(fd, EVIOCGBIT(0, sizeof(evbit)), evbit) < 0) ||
			(ioctl(fd, EVIOCGBIT(EV_REL, sizeof(relbit)), relbit) < 0) ||
			(ioctl(fd, EVIOCGBIT(EV_KEY, sizeof(keybit)), keybit) < 0) ) {
		return 0;
	}

	return 1;
}

static void InitLinuxEventDevices(void) {
        char path[PATH_MAX];
	struct stat sb;
	int duplicate, i, j, n, fd;
	int numdevices;
	dev_t dev_nums[MAX_EVDEV];
	const char *evdev_pattern[] = {
	    "/dev/input/event%d",
	    "/dtv/event%d"
	};

        numdevices = 0;

	for (i = 0; i < MAX_EVDEV; i++) {
		evdevlist[i].fd = -1;
	}

	for (i = 0; i < SDL_arraysize(evdev_pattern); i++) {
		for (j = 0; j < MAX_EVDEV; j++) {
			SDL_snprintf(path, SDL_arraysize(path), evdev_pattern[i], j);
			if (stat(path, &sb) == 0) {
				duplicate = 0;
				for (n = 0; (n < numdevices) && !duplicate; n++) {
                            		if (sb.st_rdev == dev_nums[n]) {
						duplicate = 1;
					}
				}
				if (duplicate) {
					continue;
				}

				fd = open(path, O_RDONLY);
				if (fd < 0) {
					continue;
				}

				if (!IsMouseOrKeyboard(fd)) {
					close(fd);
					continue;
				}
				fcntl(fd, F_SETFL, O_NONBLOCK);
				evdevlist[numdevices].fd = fd;
				dev_nums[numdevices] = sb.st_rdev;
				numdevices++;
			}
		}
	}
}

static SDL_keysym *TranslateEventKey(unsigned short vkey, SDL_keysym *keysym)
{
	keysym->scancode = vkey;
	keysym->sym = keymapKeyboard[vkey];
	keysym->mod = KMOD_NONE;
	keysym->unicode = 0;

	return(keysym);
}

void DUMMY_CloseEventDevices(void) {
	int j;

	for (j = 0; j < MAX_EVDEV; j++) {
		if (evdevlist[j].fd < 0)
			continue;
		close(evdevlist[j].fd);
		evdevlist[j].fd = -1;
	}
}

#endif

static SDL_keysym *TranslateKey(unsigned short vkey, SDL_keysym *keysym)
{
	keysym->scancode = vkey;
	keysym->sym = keymapRemote[vkey];
	keysym->mod = KMOD_NONE;
	keysym->unicode = 0;

	return(keysym);
}

void DUMMY_PumpEvents(_THIS)
{
	SDL_keysym keysym;

	// handle TV remote controller

	if (InputKeyState == 1) {
		SDL_PrivateKeyboard(SDL_PRESSED, TranslateKey(InputKeyData, &keysym));
	}	
	if (InputKeyState == 2) {
		SDL_PrivateKeyboard(SDL_RELEASED, TranslateKey(InputKeyData, &keysym));
	}

	InputKeyState = 0;

#if SDL_INPUT_LINUXEV

	// handle mouse and keyboard

	struct input_event ev[64];
	int i, j, len;

	for (i = 0; i < MAX_EVDEV; i++) {
		if (evdevlist[i].fd < 0) {
			continue;
		}
		while ((len = read(evdevlist[i].fd, ev, sizeof(struct input_event))) > 0) {
			len /= sizeof(struct input_event);
			for (j = 0; j < len; j++) {
				if (ev[j].type == EV_KEY) {
					if (ev[j].code == BTN_LEFT) {
						if (ev[j].value == 0)
							SDL_PrivateMouseButton(SDL_RELEASED, SDL_BUTTON_LEFT, 0, 0);
						if (ev[j].value == 1)
							SDL_PrivateMouseButton(SDL_PRESSED, SDL_BUTTON_LEFT, 0, 0);
					} else if (ev[j].code == BTN_MIDDLE) {
						if (ev[j].value == 0)
							SDL_PrivateMouseButton(SDL_RELEASED, SDL_BUTTON_MIDDLE, 0, 0);
						if (ev[j].value == 1)
							SDL_PrivateMouseButton(SDL_PRESSED, SDL_BUTTON_MIDDLE, 0, 0);
					} else if (ev[j].code == BTN_RIGHT) {
						if (ev[j].value == 0)
							SDL_PrivateMouseButton(SDL_RELEASED, SDL_BUTTON_RIGHT, 0, 0);
						if (ev[j].value == 1)
							SDL_PrivateMouseButton(SDL_PRESSED, SDL_BUTTON_RIGHT, 0, 0);
					} else {
						if (ev[j].value == 0)
							SDL_PrivateKeyboard(SDL_RELEASED, TranslateEventKey(ev[j].code, &keysym));
						if (ev[j].value == 1)
							SDL_PrivateKeyboard(SDL_PRESSED, TranslateEventKey(ev[j].code, &keysym));
					}
				} else if (ev[j].type == EV_REL) {
					if (ev[j].code == REL_X) {
						SDL_PrivateMouseMotion(0, 1, ev[j].value, 0);
					} else if (ev[j].code == REL_Y) {
						SDL_PrivateMouseMotion(0, 1, 0, ev[j].value);
					} else if (ev[j].code == REL_WHEEL) {
						if (ev[j].value == -1) {
							SDL_PrivateMouseButton(SDL_PRESSED, SDL_BUTTON_WHEELDOWN, 0, 0);
							SDL_PrivateMouseButton(SDL_RELEASED, SDL_BUTTON_WHEELDOWN, 0, 0);
						} else if (ev[j].value == 1) {
							SDL_PrivateMouseButton(SDL_PRESSED, SDL_BUTTON_WHEELUP, 0, 0);
							SDL_PrivateMouseButton(SDL_RELEASED, SDL_BUTTON_WHEELUP, 0, 0);
						}
					}
				}
			}
		}
	}
#endif
}

void DUMMY_InitOSKeymap(_THIS)
{
	memset(keymapRemote, 0, sizeof(SDLKey) * 256);

	keymapRemote[88]	= SDL_QUIT; // RETURN button
	keymapRemote[0]		= SDLK_z; // Ok button
	keymapRemote[2]		= SDLK_UP;
	keymapRemote[3]		= SDLK_DOWN;
	keymapRemote[4]		= SDLK_LEFT;
	keymapRemote[5]		= SDLK_RIGHT;
	keymapRemote[17]	= SDLK_0;
	keymapRemote[101]	= SDLK_1;
	keymapRemote[98]	= SDLK_2;
	keymapRemote[6]		= SDLK_3;
	keymapRemote[8]		= SDLK_4;
	keymapRemote[9]		= SDLK_5;
	keymapRemote[10]	= SDLK_6;
	keymapRemote[12]	= SDLK_7;
	keymapRemote[13]	= SDLK_8;
	keymapRemote[14]	= SDLK_9;
	keymapRemote[108]	= SDLK_HOME; // Red button
	keymapRemote[20]	= SDLK_F1; // Green button
	keymapRemote[21]	= SDLK_F2; // Yellow button
	keymapRemote[22]	= SDLK_ESCAPE; // Blue button
	keymapRemote[45]	= SDLK_POWER; // Exit button

#if SDL_INPUT_LINUXEV
	InitLinuxEventDevices();

	memset(keymapKeyboard, 0, sizeof(SDLKey) * KEY_MAX);

	keymapKeyboard[KEY_ESC] = SDLK_ESCAPE;
	keymapKeyboard[KEY_F1] = SDLK_F1;
	keymapKeyboard[KEY_F2] = SDLK_F2;
	keymapKeyboard[KEY_F3] = SDLK_F3;
	keymapKeyboard[KEY_F4] = SDLK_F4;
	keymapKeyboard[KEY_F5] = SDLK_F5;
	keymapKeyboard[KEY_F6] = SDLK_F6;
	keymapKeyboard[KEY_F7] = SDLK_F7;
	keymapKeyboard[KEY_F8] = SDLK_F8;
	keymapKeyboard[KEY_F9] = SDLK_F9;
	keymapKeyboard[KEY_F10] = SDLK_F10;
	keymapKeyboard[KEY_F11] = SDLK_F11;
	keymapKeyboard[KEY_F12] = SDLK_F12;
	keymapKeyboard[KEY_GRAVE] = SDLK_BACKQUOTE;
	keymapKeyboard[KEY_1] = SDLK_1;
	keymapKeyboard[KEY_2] = SDLK_2;
	keymapKeyboard[KEY_3] = SDLK_3;
	keymapKeyboard[KEY_4] = SDLK_4;
	keymapKeyboard[KEY_5] = SDLK_5;
	keymapKeyboard[KEY_6] = SDLK_6;
	keymapKeyboard[KEY_7] = SDLK_7;
	keymapKeyboard[KEY_8] = SDLK_8;
	keymapKeyboard[KEY_9] = SDLK_9;
	keymapKeyboard[KEY_0] = SDLK_0;
	keymapKeyboard[KEY_MINUS] = SDLK_MINUS;
	keymapKeyboard[KEY_EQUAL] = SDLK_EQUALS;
	keymapKeyboard[KEY_BACKSPACE] = SDLK_BACKSPACE;
	keymapKeyboard[KEY_TAB] = SDLK_TAB;
	keymapKeyboard[KEY_Q] = SDLK_q;
	keymapKeyboard[KEY_W] = SDLK_w;
	keymapKeyboard[KEY_E] = SDLK_e;
	keymapKeyboard[KEY_R] = SDLK_r;
	keymapKeyboard[KEY_T] = SDLK_t;
	keymapKeyboard[KEY_Y] = SDLK_y;
	keymapKeyboard[KEY_U] = SDLK_u;
	keymapKeyboard[KEY_I] = SDLK_i;
	keymapKeyboard[KEY_O] = SDLK_o;
	keymapKeyboard[KEY_P] = SDLK_p;
	keymapKeyboard[KEY_LEFTBRACE] = SDLK_LEFTBRACKET;
	keymapKeyboard[KEY_RIGHTBRACE] = SDLK_RIGHTBRACKET;
	keymapKeyboard[KEY_ENTER] = SDLK_RETURN;
	keymapKeyboard[KEY_CAPSLOCK] = SDLK_CAPSLOCK;
	keymapKeyboard[KEY_A] = SDLK_a;
	keymapKeyboard[KEY_S] = SDLK_s;
	keymapKeyboard[KEY_D] = SDLK_d;
	keymapKeyboard[KEY_F] = SDLK_f;
	keymapKeyboard[KEY_G] = SDLK_g;
	keymapKeyboard[KEY_H] = SDLK_h;
	keymapKeyboard[KEY_J] = SDLK_j;
	keymapKeyboard[KEY_K] = SDLK_k;
	keymapKeyboard[KEY_L] = SDLK_l;
	keymapKeyboard[KEY_SEMICOLON] = SDLK_SEMICOLON;
	keymapKeyboard[KEY_APOSTROPHE] = SDLK_QUOTE;
	keymapKeyboard[KEY_BACKSLASH] = SDLK_BACKSLASH;
	keymapKeyboard[KEY_LEFTSHIFT] = SDLK_LSHIFT;
	keymapKeyboard[KEY_Z] = SDLK_z;
	keymapKeyboard[KEY_X] = SDLK_x;
	keymapKeyboard[KEY_C] = SDLK_c;
	keymapKeyboard[KEY_V] = SDLK_v;
	keymapKeyboard[KEY_B] = SDLK_b;
	keymapKeyboard[KEY_N] = SDLK_n;
	keymapKeyboard[KEY_M] = SDLK_m;
	keymapKeyboard[KEY_COMMA] = SDLK_COMMA;
	keymapKeyboard[KEY_DOT] = SDLK_PERIOD;
	keymapKeyboard[KEY_SLASH] = SDLK_SLASH;
	keymapKeyboard[KEY_RIGHTSHIFT] = SDLK_RSHIFT;
	keymapKeyboard[KEY_LEFTCTRL] = SDLK_LCTRL;
	keymapKeyboard[KEY_LEFTALT] = SDLK_LALT;
	keymapKeyboard[KEY_LEFTMETA] = SDLK_LMETA;
	keymapKeyboard[KEY_SPACE] = SDLK_SPACE;
	keymapKeyboard[KEY_RIGHTMETA] = SDLK_RMETA;
	keymapKeyboard[KEY_RIGHTALT] = SDLK_RALT;

	keymapKeyboard[KEY_LEFT] = SDLK_LEFT;
	keymapKeyboard[KEY_UP] = SDLK_UP;
	keymapKeyboard[KEY_DOWN] = SDLK_DOWN;
	keymapKeyboard[KEY_RIGHT] = SDLK_RIGHT;

	keymapKeyboard[KEY_INSERT] = SDLK_INSERT;
	keymapKeyboard[KEY_DELETE] = SDLK_DELETE;
	keymapKeyboard[KEY_HOME] = SDLK_HOME;
	keymapKeyboard[KEY_END] = SDLK_END;
	keymapKeyboard[KEY_PAGEUP] = SDLK_PAGEUP;
	keymapKeyboard[KEY_PAGEDOWN] = SDLK_PAGEDOWN;

	keymapKeyboard[KEY_SYSRQ] = SDLK_SYSREQ;
	keymapKeyboard[KEY_SCROLLLOCK] = SDLK_SCROLLOCK;
	keymapKeyboard[KEY_PAUSE] = SDLK_PAUSE;

	keymapKeyboard[KEY_NUMLOCK] = SDLK_NUMLOCK;
	keymapKeyboard[KEY_KPSLASH] = SDLK_KP_DIVIDE;
	keymapKeyboard[KEY_KPASTERISK] = SDLK_PAUSE;
	keymapKeyboard[KEY_KPMINUS] = SDLK_KP_MINUS;
	keymapKeyboard[KEY_KPPLUS] = SDLK_KP_PLUS;
	keymapKeyboard[KEY_KPENTER] = SDLK_KP_ENTER;
	keymapKeyboard[KEY_KPDOT] = SDLK_KP_PERIOD;
	keymapKeyboard[KEY_KP0] = SDLK_KP0;
	keymapKeyboard[KEY_KP1] = SDLK_KP1;
	keymapKeyboard[KEY_KP2] = SDLK_KP2;
	keymapKeyboard[KEY_KP3] = SDLK_KP3;
	keymapKeyboard[KEY_KP4] = SDLK_KP4;
	keymapKeyboard[KEY_KP5] = SDLK_KP5;
	keymapKeyboard[KEY_KP6] = SDLK_KP6;
	keymapKeyboard[KEY_KP7] = SDLK_KP7;
	keymapKeyboard[KEY_KP8] = SDLK_KP8;
	keymapKeyboard[KEY_KP9] = SDLK_KP9;

//	keymapKeyboard[KEY_] = SDLK_;

#endif
}

/* end of SDL_nullevents.c ... */
