/*
    SDL - Simple DirectMedia Layer
    Copyright (C) 1997-2009 Sam Lantinga

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Sam Lantinga
    slouken@libsdl.org
*/
#include "SDL_config.h"

/* Dummy SDL video driver implementation; this is just enough to make an
 *  SDL-based application THINK it's got a working video driver, for
 *  applications that call SDL_Init(SDL_INIT_VIDEO) when they don't need it,
 *  and also for use as a collection of stubs when porting SDL to a new
 *  platform for which you haven't yet written a valid video driver.
 *
 * This is also a great way to determine bottlenecks: if you think that SDL
 *  is a performance problem for a given platform, enable this driver, and
 *  then see if your application runs faster without video overhead.
 *
 * Initial work by Ryan C. Gordon (icculus@icculus.org). A good portion
 *  of this was cut-and-pasted from Stephane Peter's work in the AAlib
 *  SDL video driver.  Renamed to "DUMMY" by Sam Lantinga.
 */

#include "SDL_video.h"
#include "SDL_mouse.h"
#include "../SDL_sysvideo.h"
#include "../SDL_pixels_c.h"
#include "../../events/SDL_events_c.h"

#include "SDL_nullvideo.h"
#include "SDL_nullevents_c.h"
#include "SDL_nullmouse_c.h"

#define DUMMYVID_DRIVER_NAME "SamsungTV Video"

extern char *GetFrameBufferAddress(void);
extern void VideoInit(int, int, int);
extern void VideoClose(void);
extern void UpdateSurface(char *, int, int);
extern int y_SetHWAlpha(SDL_Surface *, Uint8);
extern int y_SetHWColorKey(SDL_Surface *, Uint32);
extern int y_HWSWAccelBlit(SDL_Surface *, SDL_Rect *, SDL_Surface *, SDL_Rect *);
extern int y_HWAccelBlit(SDL_Surface *, SDL_Rect *, SDL_Surface *, SDL_Rect *);
extern int y_HWFillRect(SDL_Surface *, SDL_Rect *, Uint32);
extern int y_AllocHWSurface(SDL_Surface *);
extern void y_FreeHWSurface(SDL_Surface *);

/* Initialization/Query functions */
static int DUMMY_VideoInit(_THIS, SDL_PixelFormat *vformat);
static SDL_Rect **DUMMY_ListModes(_THIS, SDL_PixelFormat *format, Uint32 flags);
static SDL_Surface *DUMMY_SetVideoMode(_THIS, SDL_Surface *current, int width, int height, int bpp, Uint32 flags);
static int DUMMY_SetColors(_THIS, int firstcolor, int ncolors, SDL_Color *colors);
static void DUMMY_VideoQuit(_THIS);

/* Hardware surface functions */
static int DUMMY_AllocHWSurface(_THIS, SDL_Surface *surface);
static int DUMMY_LockHWSurface(_THIS, SDL_Surface *surface);
static void DUMMY_UnlockHWSurface(_THIS, SDL_Surface *surface);
static void DUMMY_FreeHWSurface(_THIS, SDL_Surface *surface);

/* etc. */
static void DUMMY_UpdateRects(_THIS, int numrects, SDL_Rect *rects);

static int DUMMY_HWFillRect(_THIS, SDL_Surface *dst, SDL_Rect *rect, Uint32 color);
static int DUMMY_CheckHWBlit(_THIS, SDL_Surface *src, SDL_Surface *dst);
static int DUMMY_HWAccelBlit(SDL_Surface *src, SDL_Rect *srcrect, SDL_Surface *dst, SDL_Rect *dstrect);
static int DUMMY_HWSWAccelBlit(SDL_Surface *src, SDL_Rect *srcrect, SDL_Surface *dst, SDL_Rect *dstrect);
static int DUMMY_SetHWColorKey(_THIS, SDL_Surface *surface, Uint32 key);
static int DUMMY_SetHWAlpha(_THIS, SDL_Surface *surface, Uint8 value);

/* DUMMY driver bootstrap functions */

static int DUMMY_Available(void)
{
	return 1;
}

static void DUMMY_DeleteDevice(SDL_VideoDevice *device)
{
	SDL_free(device->hidden);
	SDL_free(device);
}

static SDL_VideoDevice *DUMMY_CreateDevice(int devindex)
{
	SDL_VideoDevice *device;

	/* Initialize all variables that we clean on shutdown */
	device = (SDL_VideoDevice *)SDL_malloc(sizeof(SDL_VideoDevice));
	if ( device ) {
		SDL_memset(device, 0, (sizeof *device));
		device->hidden = (struct SDL_PrivateVideoData *)
				SDL_malloc((sizeof *device->hidden));
	}
	if ( (device == NULL) || (device->hidden == NULL) ) {
		SDL_OutOfMemory();
		if ( device ) {
			SDL_free(device);
		}
		return(0);
	}
	SDL_memset(device->hidden, 0, (sizeof *device->hidden));

	/* Set the function pointers */
	device->VideoInit = DUMMY_VideoInit;
	device->ListModes = DUMMY_ListModes;
	device->SetVideoMode = DUMMY_SetVideoMode;
	device->CreateYUVOverlay = NULL;
	device->SetColors = DUMMY_SetColors;
	device->UpdateRects = NULL;
	device->VideoQuit = DUMMY_VideoQuit;
	device->AllocHWSurface = DUMMY_AllocHWSurface;
	device->CheckHWBlit = DUMMY_CheckHWBlit;
	device->FillHWRect = DUMMY_HWFillRect;
	device->SetHWColorKey = DUMMY_SetHWColorKey;
	device->SetHWAlpha = DUMMY_SetHWAlpha;
	device->LockHWSurface = DUMMY_LockHWSurface;
	device->UnlockHWSurface = DUMMY_UnlockHWSurface;
	device->FlipHWSurface = NULL;
	device->FreeHWSurface = DUMMY_FreeHWSurface;
	device->SetCaption = NULL;
	device->SetIcon = NULL;
	device->IconifyWindow = NULL;
	device->GrabInput = NULL;
	device->GetWMInfo = NULL;
	device->InitOSKeymap = DUMMY_InitOSKeymap;
	device->PumpEvents = DUMMY_PumpEvents;

	device->free = DUMMY_DeleteDevice;

	return device;
}

VideoBootStrap DUMMY_bootstrap = {
	DUMMYVID_DRIVER_NAME, "SDL SamsungTV video driver",
	DUMMY_Available, DUMMY_CreateDevice
};


int DUMMY_VideoInit(_THIS, SDL_PixelFormat *vformat)
{
	vformat->Amask = 0xFF000000;
	vformat->Rmask = 0x00FF0000;
	vformat->Gmask = 0x0000FF00;
	vformat->Bmask = 0x000000FF;

	vformat->BitsPerPixel = 32;
	vformat->BytesPerPixel = 4;
	vformat->alpha = 255;

	this->info.wm_available = 0;
	this->info.hw_available = 1;
	this->info.blit_hw      = 1;
	this->info.blit_hw_CC   = 1;
	this->info.blit_hw_A    = 1;
	this->info.blit_sw      = 1;
	this->info.blit_sw_CC   = 1;
	this->info.blit_sw_A    = 1;
	this->info.blit_fill    = 1;
	this->info.current_w    = 960;
	this->info.current_h    = 540;

	/* We're done! */
	return(0);
}

SDL_Rect **DUMMY_ListModes(_THIS, SDL_PixelFormat *format, Uint32 flags)
{
   	 return (SDL_Rect **) -1;
}

SDL_Surface *DUMMY_SetVideoMode(_THIS, SDL_Surface *current,
				int width, int height, int bpp, Uint32 flags)
{
	if (width > 960 || height > 540) {
		SDL_SetError("Resolution bigger then 960x540 not supported");
		return(NULL);
	}

	bpp = 32; // force to 32 bit
	VideoInit(width, height, 32);
	this->hidden->buffer = GetFrameBufferAddress();
	if ( ! this->hidden->buffer) {
		SDL_SetError("Couldn't allocate buffer for requested mode");
		return(NULL);
	}

	SDL_memset(this->hidden->buffer, 0, width * height * (bpp / 8));

	/* Allocate the new pixel format for the screen */
	if ( ! SDL_ReallocFormat(current, bpp, 0x00FF0000, 0x0000FF00, 0x000000FF, 0xFF000000) ) {
		this->hidden->buffer = NULL;
		SDL_SetError("Couldn't allocate new pixel format for requested mode");
		return(NULL);
	}

	/* Set up the new mode framebuffer */
	current->flags = flags | SDL_FULLSCREEN;
	this->hidden->w = current->w = width;
	this->hidden->h = current->h = height;
	this->hidden->bpp = bpp;
	current->pitch = current->w * (bpp / 8);
	current->pixels = this->hidden->buffer;
	this->UpdateRects = DUMMY_UpdateRects;

	/* We're done */
	return(current);
}

/* We don't actually allow hardware surfaces other than the main one */
static int DUMMY_AllocHWSurface(_THIS, SDL_Surface *surface)
{
	return y_AllocHWSurface(surface);
}

static void DUMMY_FreeHWSurface(_THIS, SDL_Surface *surface)
{
	y_FreeHWSurface(surface);
}

/* We need to wait for vertical retrace on page flipped displays */
static int DUMMY_LockHWSurface(_THIS, SDL_Surface *surface)
{
	return(0);
}

static void DUMMY_UnlockHWSurface(_THIS, SDL_Surface *surface)
{
	return;
}

static void DUMMY_UpdateRects(_THIS, int numrects, SDL_Rect *rects)
{
	UpdateSurface((char *)this->hidden->buffer, this->hidden->w, this->hidden->h);
}

int DUMMY_SetColors(_THIS, int firstcolor, int ncolors, SDL_Color *colors)
{
	/* do nothing of note. */
	return(1);
}

/* Note:  If we are terminated, this could be called in the middle of
   another SDL video routine -- notably UpdateRects.
*/

void DUMMY_CloseEventDevices(void);

void DUMMY_VideoQuit(_THIS)
{
	SDL_memset(this->hidden->buffer, 0, this->hidden->w * this->hidden->h * (this->hidden->bpp / 8));
	this->screen->pixels = NULL;
	this->hidden->buffer = NULL;
	DUMMY_CloseEventDevices();
}

int DUMMY_HWFillRect(_THIS, SDL_Surface *dst, SDL_Rect *rect, Uint32 color)
{
	return y_HWFillRect(dst, rect, color);
}

int DUMMY_CheckHWBlit(_THIS, SDL_Surface *src, SDL_Surface *dst)
{
	if (src->format->BitsPerPixel != this->hidden->bpp || dst->format->BitsPerPixel != this->hidden->bpp)
		return(-1);

	if ((src->flags & SDL_HWSURFACE) != SDL_HWSURFACE && (dst->flags & SDL_HWSURFACE) != SDL_HWSURFACE)
	{
		return(-1);
	}

	src->flags |= SDL_HWACCEL;

	if ((src->flags & SDL_SRCCOLORKEY ) == SDL_SRCCOLORKEY)
	{
		src->flags &= ~SDL_HWACCEL;
	}

	if ((src->flags & SDL_SRCALPHA) == SDL_SRCALPHA)
	{
		// TODO
	}

	if ((src->flags & SDL_HWSURFACE) == SDL_HWSURFACE && (dst->flags & SDL_HWSURFACE) == SDL_HWSURFACE)
	{
		src->map->hw_blit = DUMMY_HWAccelBlit;
		return 1;
	}

	else if ((src->flags & SDL_SWSURFACE) == SDL_SWSURFACE && (dst->flags & SDL_HWSURFACE) == SDL_HWSURFACE)
	{
		src->map->hw_blit = DUMMY_HWSWAccelBlit;
		return 1;
	}

	return (-1);
}

int DUMMY_HWAccelBlit(SDL_Surface *src, SDL_Rect *srcrect, SDL_Surface *dst, SDL_Rect *dstrect)
{
	return y_HWAccelBlit(src, srcrect, dst, dstrect);
}

int DUMMY_HWSWAccelBlit(SDL_Surface *src, SDL_Rect *srcrect, SDL_Surface *dst, SDL_Rect *dstrect)
{
	return y_HWSWAccelBlit(src, srcrect, dst, dstrect);
}

int DUMMY_SetHWColorKey(_THIS, SDL_Surface *surface, Uint32 key)
{
	return y_SetHWColorKey(surface, key);
}

int DUMMY_SetHWAlpha(_THIS, SDL_Surface *surface, Uint8 value)
{
	return y_SetHWAlpha(surface, value);
}
